from abc import ABC, abstractmethod
import logging
from crllm.config.config_service import config_service


class Model(ABC):
    def generate(self, prompt_template, prompt_args):
        config = config_service.get_config()
        model_config = config["model"]

        model = self._get_model(model_config)

        llm_chain = prompt_template | model
        result = llm_chain.invoke(prompt_args)

        logging.debug(result)

        return result.content

    @abstractmethod
    def _get_model(self, model_config):
        pass



from langchain_huggingface import ChatHuggingFace, HuggingFaceEndpoint
from crllm.model.model import Model


class HuggingFaceModel(Model):
    def _get_model(self, model_config):
        llm = HuggingFaceEndpoint(**model_config)

        return ChatHuggingFace(llm=llm)



import logging
from langchain_core.prompts import ChatPromptTemplate
from crllm.config.config_service import config_service


class PromptBuilder:
    def build(self):
        config = config_service.get_config()

        logging.debug(config["prompt"])

        prompt = ChatPromptTemplate.from_messages(
            [
                ("system", config["prompt"]["template"]),
                ("human", "Do code review for the following code: \n {code}"),
            ]
        )

        return prompt



import sys
from crllm.cli import cli


def test_cli(monkeypatch, capfd):

    file_path = "./src/crllm/test/functional/smoke_test.py"
    config_path = "./src/crllm/test/functional/config.toml"

    monkeypatch.setattr(sys, "argv", ["crllm", file_path, "-c", config_path])
    cli()
    out, err = capfd.readouterr()

    assert not err
    assert "This is a dry run!" in out
